@@ -1,11 +1,9 @@
 /*
- * $Id: heap.h,v 1.15 2003/07/01 13:18:33 erik Exp $
- *
  * heap.h -- generic heap
  *
  * Alexis Yushin, <alexis@nlnetlabs.nl>
  *
- * Copyright (c) 2001, 2002, 2003, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2004, NLnet Labs. All rights reserved.
  *
  * This software is an open source.
  *
@@ -43,15 +41,17 @@
 
 #include "rbtree.h"
 
-#define	heap_t	rbtree_t
+#define	heap_t	        rbtree_t
+#define heapnode_t      rbnode_t
 #define	heap_create	rbtree_create
 #define	heap_insert	rbtree_insert
 #define	heap_search	rbtree_search
 #define	heap_delete	rbtree_delete
-#define	heap_destroy	rbtree_destroy
 #define	heap_first	rbtree_first
-#define	heap_next	rbtree_next
 #define	heap_last	rbtree_last
+#define	heap_next	rbtree_next
+#define	heap_previous	rbtree_previous
 #define	HEAP_WALK	RBTREE_WALK
+#define HEAP_NULL       RBTREE_NULL
 
 #endif /* _HEAP_H_ */
