@@ -46,7 +46,7 @@
 #include "heap.h"
 #include "region-allocator.h"
 
-#define	NAMEDB_MAGIC		"NSDdbV05"
+#define	NAMEDB_MAGIC		"NSDdbV06"
 #define	NAMEDB_MAGIC_SIZE	8
 
 #if defined(NAMEDB_UPPERCASE) || defined(USE_NAMEDB_UPPERCASE)
@@ -56,7 +56,7 @@
 #endif
 
 
-typedef struct rdata_atom rdata_atom_type;
+typedef union rdata_atom rdata_atom_type;
 typedef struct rrset rrset_type;
 typedef struct rrdata rrdata_type;
 
@@ -94,7 +94,7 @@ struct domain
 struct zone
 {
 	zone_type   *next;
-	domain_type *domain;
+	domain_type *apex;
 	rrset_type  *soa_rrset;
 	rrset_type  *ns_rrset;
 	uint32_t     number;
@@ -106,19 +106,29 @@ struct rrset
 	rrset_type   *next;
 	zone_type    *zone;
 	uint16_t      type;
-	uint16_t      class;
+	uint16_t      klass;	/* Avoid conflict with C++ keyword.  */
 	uint16_t      rrslen;
 	rrdata_type **rrs;
 };
 
-struct rdata_atom
+/*
+ * The field used is based on the wireformat the atom is stored in.
+ * The allowed wireformats are defined by the rdata_wireformat_type
+ * enumeration.
+ */
+union rdata_atom
 {
-	void *data;
+	/* RDATA_WF_COMPRESSED_DNAME, RDATA_WF_UNCOMPRESSED_DNAME.  */
+	domain_type *domain;
+
+	/* Default.  */
+	uint16_t    *data;
 };
 
 struct rrdata
 {
 	int32_t         ttl;
+	uint16_t        rdata_count;
 	rdata_atom_type rdata[1];
 };
 
@@ -196,7 +206,7 @@ int zone_is_secure(zone_type *zone);
 static inline const dname_type *
 domain_dname(domain_type *domain)
 {
-	return domain->node.key;
+	return (const dname_type *) domain->node.key;
 }
 
 
@@ -216,45 +226,38 @@ struct namedb
 	FILE              *fd;
 };
 
-static inline int
-rdata_atom_is_terminator(rdata_atom_type atom)
-{
-	return atom.data == NULL;
-}
-
 static inline int rdata_atom_is_domain(uint16_t type, size_t index);
 
-static inline int
-rdata_atom_is_data(uint16_t type, size_t index)
-{
-	return !rdata_atom_is_domain(type, index);
-}
-
 static inline domain_type *
 rdata_atom_domain(rdata_atom_type atom)
 {
-	return (domain_type *) atom.data;
+	return atom.domain;
 }
 
 static inline uint16_t
 rdata_atom_size(rdata_atom_type atom)
 {
-	return * (uint16_t *) atom.data;
+	return *atom.data;
 }
 
 static inline void *
 rdata_atom_data(rdata_atom_type atom)
 {
-	return (uint16_t *) atom.data + 1;
+	return atom.data + 1;
 }
 
 static inline size_t
 rrdata_size(uint16_t rdcount)
 {
-	return sizeof(rrdata_type) + rdcount * sizeof(rdata_atom_type);
+	return sizeof(rrdata_type) + (rdcount - 1) * sizeof(rdata_atom_type);
 }
 
 
+/*
+ * Find the zone for the specified DOMAIN in DB.
+ */
+zone_type *namedb_find_zone(namedb_type *db, domain_type *domain);
+
 /* dbcreate.c */
 struct namedb *namedb_new(const char *filename);
 int namedb_save(struct namedb *db);
@@ -269,12 +272,23 @@ int namedb_lookup (struct namedb    *db,
 struct namedb *namedb_open(const char *filename);
 void namedb_close(struct namedb *db);
 
-extern const char *rdata_types[];
-
 static inline int
 rdata_atom_is_domain(uint16_t type, size_t index)
 {
-	return type < TYPE_TXT && index < 2 && rdata_types[type][index] == 'd';
+	const rrtype_descriptor_type *descriptor
+		= rrtype_descriptor_by_type(type);
+	return (index < descriptor->maximum
+		&& (descriptor->wireformat[index] == RDATA_WF_COMPRESSED_DNAME
+		    || descriptor->wireformat[index] == RDATA_WF_UNCOMPRESSED_DNAME));
+}
+
+static inline rdata_wireformat_type
+rdata_atom_wireformat_type(uint16_t type, size_t index)
+{
+	const rrtype_descriptor_type *descriptor
+		= rrtype_descriptor_by_type(type);
+	assert(index < descriptor->maximum);
+	return (rdata_wireformat_type) descriptor->wireformat[index];
 }
 
 #endif
