@@ -249,7 +249,7 @@ restart_child_servers(struct nsd *nsd)
 static void
 initialize_dname_compression_tables(struct nsd *nsd)
 {
-	compressed_dname_offsets = xalloc(
+	compressed_dname_offsets = (uint16_t *) xalloc(
 		(domain_table_count(nsd->db->domains) + 1) * sizeof(uint16_t));
 	memset(compressed_dname_offsets, 0,
 	       (domain_table_count(nsd->db->domains) + 1) * sizeof(uint16_t));
@@ -266,7 +266,7 @@ int
 server_init(struct nsd *nsd)
 {
 	size_t i;
-#if defined(SO_REUSEADDR) || (defined(INET6) && defined(IPV6_V6ONLY))
+#if defined(SO_REUSEADDR) || (defined(INET6) && (defined(IPV6_V6ONLY) || defined(IPV6_USE_MIN_MTU)))
 	int on = 1;
 #endif
 
@@ -279,13 +279,36 @@ server_init(struct nsd *nsd)
 			return -1;
 		}
 
-#if defined(INET6) && defined(IPV6_V6ONLY)
-		if (nsd->udp[i].addr->ai_family == AF_INET6 &&
-		    setsockopt(nsd->udp[i].s, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on)) < 0)
-		{
-			log_msg(LOG_ERR, "setsockopt(..., IPV6_V6ONLY, ...) failed: %s",
-				strerror(errno));
-			return -1;
+#if defined(INET6)
+		if (nsd->udp[i].addr->ai_family == AF_INET6) {
+# if defined(IPV6_V6ONLY)
+			if (setsockopt(nsd->udp[i].s,
+				       IPPROTO_IPV6, IPV6_V6ONLY,
+				       &on, sizeof(on)) < 0)
+			{
+				log_msg(LOG_ERR, "setsockopt(..., IPV6_V6ONLY, ...) failed: %s",
+					strerror(errno));
+				return -1;
+			}
+# endif
+# if defined(IPV6_USE_MIN_MTU)
+			/*
+			 * There is no fragmentation of IPv6 datagrams
+			 * during forwarding in the network. Therefore
+			 * we do not send UDP datagrams larger than
+			 * the minimum IPv6 MTU of 1280 octets. The
+			 * EDNS0 message length can be larger if the
+			 * network stack supports IPV6_USE_MIN_MTU.
+			 */
+			if (setsockopt(nsd->udp[i].s,
+				       IPPROTO_IPV6, IPV6_USE_MIN_MTU,
+				       &on, sizeof(on)) < 0)
+			{
+				log_msg(LOG_ERR, "setsockopt(..., IPV6_USE_MIN_MTU, ...) failed: %s",
+					strerror(errno));
+				return -1;
+			}
+# endif
 		}
 #endif
 
@@ -623,6 +646,7 @@ server_child(struct nsd *nsd)
 	sigemptyset(&block_sigmask);
 	sigaddset(&block_sigmask, SIGHUP);
 	sigaddset(&block_sigmask, SIGILL);
+	sigaddset(&block_sigmask, SIGUSR1);
 	sigaddset(&block_sigmask, SIGINT);
 	sigaddset(&block_sigmask, SIGTERM);
 	sigprocmask(SIG_BLOCK, &block_sigmask, &default_sigmask);
@@ -632,12 +656,15 @@ server_child(struct nsd *nsd)
 			struct udp_handler_data *data;
 			netio_handler_type *handler;
 
-			data = region_alloc(server_region, sizeof(struct udp_handler_data));
+			data = (struct udp_handler_data *) region_alloc(
+				server_region,
+				sizeof(struct udp_handler_data));
 			data->query_region = query_region;
 			data->nsd = nsd;
 			data->socket = &nsd->udp[i];
 
-			handler = region_alloc(server_region, sizeof(netio_handler_type));
+			handler = (netio_handler_type *) region_alloc(
+				server_region, sizeof(netio_handler_type));
 			handler->fd = nsd->udp[i].s;
 			handler->timeout = NULL;
 			handler->user_data = data;
@@ -652,14 +679,16 @@ server_child(struct nsd *nsd)
 	 * and disable them based on the current number of active TCP
 	 * connections.
 	 */
-	tcp_accept_handlers = region_alloc(server_region,
-					   nsd->ifs * sizeof(netio_handler_type));
+	tcp_accept_handlers = (netio_handler_type *) region_alloc(
+		server_region, nsd->ifs * sizeof(netio_handler_type));
 	if (nsd->server_kind & NSD_SERVER_TCP) {
 		for (i = 0; i < nsd->ifs; ++i) {
 			struct tcp_accept_handler_data *data;
 			netio_handler_type *handler;
 			
-			data = region_alloc(server_region, sizeof(struct tcp_accept_handler_data));
+			data = (struct tcp_accept_handler_data *) region_alloc(
+				server_region,
+				sizeof(struct tcp_accept_handler_data));
 			data->query_region = query_region;
 			data->nsd = nsd;
 			data->socket = &nsd->tcp[i];
@@ -722,7 +751,8 @@ handle_udp(netio_type *netio ATTR_UNUSED,
 	   netio_handler_type *handler,
 	   netio_event_types_type event_types)
 {
-	struct udp_handler_data *data = handler->user_data;
+	struct udp_handler_data *data
+		= (struct udp_handler_data *) handler->user_data;
 	int received, sent;
 	struct query q;
 
@@ -785,7 +815,8 @@ handle_udp(netio_type *netio ATTR_UNUSED,
 static void
 cleanup_tcp_handler(netio_type *netio, netio_handler_type *handler)
 {
-	struct tcp_handler_data *data = handler->user_data;
+	struct tcp_handler_data *data
+		= (struct tcp_handler_data *) handler->user_data;
 	netio_remove_handler(netio, handler);
 	close(handler->fd);
 
@@ -810,7 +841,8 @@ handle_tcp_reading(netio_type *netio,
 		   netio_handler_type *handler,
 		   netio_event_types_type event_types)
 {
-	struct tcp_handler_data *data = handler->user_data;
+	struct tcp_handler_data *data
+		= (struct tcp_handler_data *) handler->user_data;
 	ssize_t received;
 	struct query *q = &data->query;
 
@@ -909,10 +941,22 @@ handle_tcp_reading(netio_type *netio,
 
 	assert(query_used_size(q) == q->tcplen);
 
+	/* Account... */
+#ifndef INET6
+	STATUP(data->nsd, ctcp);
+#else
+	if (data->query.addr.ss_family == AF_INET) {
+		STATUP(data->nsd, ctcp);
+	} else if (data->query.addr.ss_family == AF_INET6) {
+		STATUP(data->nsd, ctcp6);
+	}
+#endif
+
 	/* We have a complete query, process it.  */
 	data->query_state = process_query(data->nsd, q);
 	if (data->query_state == QUERY_DISCARDED) {
-		/* Drop the entire connection... */
+		/* Drop the packet and the entire connection... */
+		STATUP(data->nsd, dropped);
 		cleanup_tcp_handler(netio, handler);
 		return;
 	}
@@ -940,7 +984,8 @@ handle_tcp_writing(netio_type *netio,
 		   netio_handler_type *handler,
 		   netio_event_types_type event_types)
 {
-	struct tcp_handler_data *data = handler->user_data;
+	struct tcp_handler_data *data
+		= (struct tcp_handler_data *) handler->user_data;
 	ssize_t sent;
 	struct query *q = &data->query;
 
@@ -1064,7 +1109,8 @@ handle_tcp_accept(netio_type *netio,
 		  netio_handler_type *handler,
 		  netio_event_types_type event_types)
 {
-	struct tcp_accept_handler_data *data = handler->user_data;
+	struct tcp_accept_handler_data *data
+		= (struct tcp_accept_handler_data *) handler->user_data;
 	int s;
 	struct tcp_handler_data *tcp_data;
 	region_type *tcp_region;
@@ -1084,13 +1130,6 @@ handle_tcp_accept(netio_type *netio,
 		return;
 	}
 	
-	/* Account... */
-	if (data->socket->addr->ai_family == AF_INET) {
-		STATUP(data->nsd, ctcp);
-	} else if (data->socket->addr->ai_family == AF_INET6) {
-		STATUP(data->nsd, ctcp6);
-	}
-
 	/* Accept it... */
 	addrlen = sizeof(addr);
 	if ((s = accept(handler->fd, (struct sockaddr *)&addr, &addrlen)) == -1) {
@@ -1110,7 +1149,8 @@ handle_tcp_accept(netio_type *netio,
 	 * closed by the TCP handler.
 	 */
 	tcp_region = region_create(xalloc, free);
-	tcp_data = region_alloc(tcp_region, sizeof(struct tcp_handler_data));
+	tcp_data = (struct tcp_handler_data *) region_alloc(
+		tcp_region, sizeof(struct tcp_handler_data));
 	tcp_data->region = tcp_region;
 	tcp_data->nsd = data->nsd;
 	
@@ -1130,9 +1170,11 @@ handle_tcp_accept(netio_type *netio,
 	memcpy(&tcp_data->query.addr, &addr, addrlen);
 	tcp_data->query.addrlen = addrlen;
 	
-	tcp_handler = region_alloc(tcp_region, sizeof(netio_handler_type));
+	tcp_handler = (netio_handler_type *) region_alloc(
+		tcp_region, sizeof(netio_handler_type));
 	tcp_handler->fd = s;
-	tcp_handler->timeout = region_alloc(tcp_region, sizeof(struct timespec));
+	tcp_handler->timeout = (struct timespec *) region_alloc(
+		tcp_region, sizeof(struct timespec));
 	tcp_handler->timeout->tv_sec = TCP_TIMEOUT;
 	tcp_handler->timeout->tv_nsec = 0L;
 	timespec_add(tcp_handler->timeout, netio_current_time(netio));
