@@ -13,7 +13,6 @@
 #include "dns.h"
 #include "namedb.h"
 
-#define	MAXRDATALEN	64		/* This is more than enough, think multiple TXT */
 #define	MAXTOKENSLEN	512		/* Maximum number of tokens per entry */
 #define	B64BUFSIZE	16384		/* Buffer size for b64 conversion */
 #define	ROOT		(const uint8_t *)"\001"
@@ -36,16 +35,15 @@ struct lex_data {
     char    *str;		/* holds the data */
 };
 
-
 #define DEFAULT_TTL 3600
 #define MAXINCLUDES 10
 
 /* a RR in DNS */
 typedef struct rr rr_type;
 struct rr {
-	domain_type *domain;
+	domain_type *owner;
 	zone_type   *zone;
-	uint16_t     class;
+	uint16_t     klass;
 	uint16_t     type;
 	rrdata_type *rrdata;
 };
@@ -53,27 +51,31 @@ struct rr {
 /* administration struct */
 typedef struct zparser zparser_type;
 struct zparser {
+	region_type *region;	/* Allocate for parser lifetime data.  */
+	region_type *rr_region;	/* Allocate RR lifetime data.  */
 	namedb_type *db;
-	int32_t ttl;
-	int32_t minimum;
-	uint16_t class;
+
+	const char *filename;
+	int32_t default_ttl;
+	int32_t default_minimum;
+	uint16_t default_class;
 	zone_type *current_zone;
 	domain_type *origin;
 	domain_type *prev_dname;
-	unsigned int _rc;   /* current rdata cnt */
+
+	int error_occurred;
 	unsigned int errors;
 	unsigned int line;
-	const char *filename;
+
+	rr_type current_rr;
+	rrdata_type *temporary_rrdata;
 };
 
-extern zparser_type *current_parser;
-extern rr_type *current_rr;
-extern rrdata_type *temporary_rrdata;
-extern int error_occurred;   /*  Error occurred while parsin an RR. */
+extern zparser_type *parser;
+
 /* used in zonec.lex */
 extern FILE *yyin;
 
-
 /*
  * Used to mark bad domains and domain names.  Do not dereference
  * these pointers!
@@ -89,121 +91,66 @@ void yyrestart(FILE *);
 enum rr_spot { outside, expecting_dname, after_dname, reading_type };
 
 /* A generic purpose lookup table */
-struct ztab {
-	uint16_t sym;
+typedef struct lookup_table lookup_table_type;
+struct lookup_table {
+	uint16_t symbol;
 	const char *name;
+	int token;		/* Lexical token ID.  */
 };
 
-#define	Z_CLASSES {		\
-	{CLASS_IN, "IN"},	\
-	{0, NULL}		\
-}
-
-#define	Z_TYPES {		\
-	{TYPE_A, "A"},		\
-	{TYPE_NS, "NS"},	\
-	{TYPE_MD, "MD"},	\
-	{TYPE_MF, "MF"},	\
-	{TYPE_CNAME, "CNAME"},	\
-	{TYPE_SOA, "SOA"},	\
-	{TYPE_MB, "MB"},	\
-	{TYPE_MG, "MG"},	\
-	{TYPE_MR, "MR"},	\
-	{TYPE_NULL, "NULL"},	\
-	{TYPE_WKS, "WKS"},	\
-	{TYPE_PTR, "PTR"},	\
-	{TYPE_HINFO, "HINFO"},	\
-	{TYPE_MINFO, "MINFO"},	\
-	{TYPE_MX, "MX"},	\
-	{TYPE_TXT, "TXT"},	\
-        {TYPE_AAAA, "AAAA"},	\
-	{TYPE_SRV, "SRV"},	\
-	{TYPE_NAPTR, "NAPTR"},	\
-	{TYPE_LOC, "LOC"},	\
-	{TYPE_AFSDB, "AFSDB"},	\
-	{TYPE_RP, "RP"},	\
-	{TYPE_SIG, "SIG"},	\
-	{TYPE_KEY, "KEY"},	\
-	{TYPE_NXT, "NXT"},	\
-	{TYPE_DS, "DS"},	\
-	{TYPE_SSHFP, "SSHFP"},  \
-	{TYPE_RRSIG, "RRSIG"},	\
-	{TYPE_NSEC, "NSEC"},	\
-	{TYPE_DNSKEY, "DNSKEY"},\
-	{TYPE_ANY, "ANY"},	\
-	{0, NULL}		\
-}
-
-#define Z_ALGS	{		\
-	{1,	"RSAMD5"},	\
-	{2,	"DS"},		\
-	{3,	"DSA"},		\
-	{4,	"ECC"},		\
-	{5,	"RSASHA1"},	\
-	{252,	"INDIRECT"},	\
-	{253,	"PRIVATEDNS"},	\
-	{254,	"PRIVATEOID"},	\
-	{0,	NULL}		\
-}
-
-extern struct ztab ztypes[];
-extern struct ztab zclasses[];
-extern struct ztab zalgs[];
-
-/* zonec.c */
-/*
- * This region is deallocated after each zone is parsed and analyzed.
- */
-extern region_type *zone_region;
-
-/*
- * This region is deallocated after each RR is parsed and analyzed.
- */
-extern region_type *rr_region;
-
 void warning(const char *fmt, ...);
 void warning_prev_line(const char *fmt, ...);
 void error(const char *fmt, ...);
 void error_prev_line(const char *fmt, ...);
 int yyerror(const char *message); /* Dummy function.  */
 
-int process_rr(zparser_type *parser, rr_type *rr);
+int process_rr(void);
 uint16_t *zparser_conv_hex(region_type *region, const char *hex);
 uint16_t *zparser_conv_time(region_type *region, const char *time);
-uint16_t *zparser_conv_rdata_proto(region_type *region, const char *protostr);
-uint16_t *zparser_conv_rdata_service(region_type *region, const char *servicestr, const int arg);
-uint16_t *zparser_conv_rdata_period(region_type *region, const char *periodstr);
+uint16_t *zparser_conv_protocol(region_type *region, const char *protostr);
+uint16_t *zparser_conv_services(region_type *region, const char *proto, char *servicestr);
+uint16_t *zparser_conv_period(region_type *region, const char *periodstr);
 uint16_t *zparser_conv_short(region_type *region, const char *shortstr);
 uint16_t *zparser_conv_long(region_type *region, const char *longstr);
 uint16_t *zparser_conv_byte(region_type *region, const char *bytestr);
 uint16_t *zparser_conv_a(region_type *region, const char *a);
-uint16_t *zparser_conv_text(region_type *region, const char *txt);
+uint16_t *zparser_conv_text(region_type *region, const char *txt, size_t len);
 uint16_t *zparser_conv_a6(region_type *region, const char *a6);
 uint16_t *zparser_conv_b64(region_type *region, const char *b64);
 uint16_t *zparser_conv_rrtype(region_type *region, const char *rr);
 uint16_t *zparser_conv_nxt(region_type *region, uint8_t nxtbits[]);
-uint16_t *zparser_conv_domain(region_type *region, domain_type *domain);
 uint16_t *zparser_conv_nsec(region_type *region, uint8_t nsecbits[NSEC_WINDOW_COUNT][NSEC_WINDOW_BITS_SIZE]);
 uint16_t *zparser_conv_loc(region_type *region, char *str);
 uint16_t *zparser_conv_algorithm(region_type *region, const char *algstr);
+uint16_t *zparser_conv_certificate_type(region_type *region,
+					const char *typestr);
+uint16_t *zparser_conv_apl_rdata(region_type *region, char *str);
+
+void parse_unknown_rdata(uint16_t type, uint16_t *wireformat);
 
 long strtottl(char *nptr, char **endptr);
 
 int32_t zparser_ttl2int(char *ttlstr);
-void zadd_rdata_wireformat(zparser_type *parser, uint16_t *data);
-void zadd_rdata_domain(zparser_type *parser, domain_type *domain);
-void zadd_rdata_finalize(zparser_type *parser);
-uint16_t intbyname (const char *a, struct ztab *tab);
-const char * namebyint (uint16_t n, struct ztab *tab);
+void zadd_rdata_wireformat(uint16_t *data);
+void zadd_rdata_domain(domain_type *domain);
 void zprintrr(FILE *f, rr_type *rr);
 
-void set_bit(uint8_t bits[], int index);
-void set_bitnsec(uint8_t bits[NSEC_WINDOW_COUNT][NSEC_WINDOW_BITS_SIZE], int index);
+void set_bit(uint8_t bits[], uint16_t index);
+void set_bitnsec(uint8_t  bits[NSEC_WINDOW_COUNT][NSEC_WINDOW_BITS_SIZE],
+		 uint16_t index);
 
-/* zlexer.lex */
-int zoctet(char *word);
-int zrrtype (char *word);
-uint16_t intbyclassxx(void *str);
-uint16_t intbytypexx(void *str);
+uint16_t intbytypexx(const char *str);
+
+const lookup_table_type *lookup_by_name(const char *a, const lookup_table_type tab[]);
+const lookup_table_type *lookup_by_symbol(uint16_t n, const lookup_table_type tab[]);
+const lookup_table_type *lookup_by_token(int token, const lookup_table_type tab[]);
+
+uint16_t lookup_type_by_name(const char *name);
+
+/* zparser.y */
+zparser_type *zparser_create(region_type *region, region_type *rr_region,
+			     namedb_type *db);
+void zparser_init(const char *filename, uint32_t ttl, uint16_t klass,
+		  const char *origin);
 
 #endif /* _ZONEC_H_ */
